/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.jobnet.dataprovider;

import java.io.File;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.prefs.Preferences;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.widgets.Table;
import org.hibernate.Session;

import com.sos.hibernate.classes.DbItem;
import com.sos.hibernate.interfaces.ISOSDashboardDataProvider;
import com.sos.hibernate.interfaces.ISOSHibernateDataProvider;
import com.sos.jobnet.classes.SosJobNetTableItem;
import com.sos.jobnet.db.JobNetNodeDBItem;
import com.sos.jobnet.db.JobNetNodeDBLayer;
import com.sos.jobnet.db.JobNetNodeFilter;

import org.apache.log4j.Logger;


/**
 * \class JobNetDataProvider
 * 
 * \brief JobNetDataProvider -
 * 
 * \details
 * 
 * \section JobNetDataProvider.java_intro_sec Introduction
 * 
 * \section JobNetDataProvider.java_samples Some Samples
 * 
 * \code .... code goes here ... \endcode
 * 
 * <p style="text-align:center">
 * <br />
 * --------------------------------------------------------------------------- <br />
 * APL/Software GmbH - Berlin <br />
 * ##### generated by ClaviusXPress (http://www.sos-berlin.com) ######### <br />
 * ---------------------------------------------------------------------------
 * </p>
 * \author Uwe Risse \version 19.01.2012 \see reference
 * 
 * Created on 19.01.2012 09:31:01
 */

public class JobNetDataProvider implements ISOSHibernateDataProvider, ISOSDashboardDataProvider {

    @SuppressWarnings("unused")
    private final String conClassName = "JobNetDataProvider";

    private List<JobNetNodeDBItem> listOfJobNetNodeDBItems = null;
    private JobNetNodeDBLayer jobNetNodeDBLayer = null;
    private static Logger logger = Logger.getLogger(JobNetDataProvider.class);
    private Table tableJobNet = null;

    public JobNetDataProvider(File configurationFile, String schedulerId) {
        this.jobNetNodeDBLayer = new JobNetNodeDBLayer(configurationFile, schedulerId);
    }

    public JobNetDataProvider(File configurationFile) {
        this.jobNetNodeDBLayer = new JobNetNodeDBLayer(configurationFile);
    }
    
    public JobNetNodeFilter getFilter() {
        return jobNetNodeDBLayer.getFilter();
    }

    public void resetFilter() {
        jobNetNodeDBLayer.setFilter(new JobNetNodeFilter());
    }

    public void getData() {
        listOfJobNetNodeDBItems = jobNetNodeDBLayer.getJobnetList(0);
    }

    public void fillSchedulerIds(CCombo cbSchedulerId) {
        if (tableJobNet != null) {
            Iterator <JobNetNodeDBItem> jobNetPlanEntries = listOfJobNetNodeDBItems.iterator();
            while (jobNetPlanEntries.hasNext()) {
                JobNetNodeDBItem h = (JobNetNodeDBItem) jobNetPlanEntries.next();
                if (cbSchedulerId.indexOf(h.getSchedulerId()) < 0) {
                    logger.debug("... cbSchedulerId --> : " + h.getSchedulerId());
                    cbSchedulerId.add(h.getSchedulerId());
                }
            }
        }
    }

   

    public void fillTable(Table table) {
        this.tableJobNet = table;
        Iterator<JobNetNodeDBItem> jobnetEntries = listOfJobNetNodeDBItems.iterator();
        while (jobnetEntries.hasNext()) {
            JobNetNodeDBItem h = jobnetEntries.next();
            if (jobNetNodeDBLayer.getFilter().isFiltered(h)) {
            } else {

                final SosJobNetTableItem newItemTableItem = new SosJobNetTableItem(table, SWT.BORDER);
                newItemTableItem.setDBItem(h);
                newItemTableItem.setData(h);
                newItemTableItem.setColor();
                newItemTableItem.setColumns();
            }
        }
    }

    public File getConfigurationFile() {
        return jobNetNodeDBLayer.getConfigurationFile();

    }

    @Override
    public void beginTransaction() {
        jobNetNodeDBLayer.beginTransaction();
    }

    public void save(JobNetNodeDBItem dbItem) {
        jobNetNodeDBLayer.save(dbItem);
    }

    @Override
    public void update(DbItem dbItem) {
        jobNetNodeDBLayer.update(dbItem);
    }

    public Session getSession() {
        return jobNetNodeDBLayer.getSession();
    }

    @Override
    public void commit() {
        jobNetNodeDBLayer.commit();
    }

    @Override
    public void setSchedulerId(String schedulerId) {
        this.getFilter().setSchedulerId(schedulerId);
    }

    @Override
    public void setFrom(Date d) {
    }

    @Override
    public void setTo(Date d) {
    }

    @Override
    public void setSearchField(String s) {
        this.getFilter().setSearchFiled(s);
    }

    @Override
    public void setOnlyJobs(boolean b) {
     }

    @Override
    public void setOnlyJobChains(boolean b) {
     }

    @Override
    public void setIgnoreList(Preferences prefs) {
     }

    @Override
    public void addToIgnorelist(Preferences prefs, DbItem h) {
     }

    @Override
    public void disableIgnoreList(Preferences prefs) {
     }

    @Override
    public void resetIgnoreList(Preferences prefs) {
     }

    @Override
    public void setLate(boolean b) {
     }

    @Override
    public void setStatus(String status) {
     }

    @Override
    public void setOnlyWithError(boolean b) {
        // TODO Auto-generated method stub
        
    }

	@Override
	public void setOnlyRunning(boolean b) {
		// TODO Auto-generated method stub
		
	}


}
