/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.jobnet.graph;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;

import com.sos.jobnet.db.JobNetEdgesDBItem;
import com.sos.jobnet.db.JobNetEdgesDBLayer;
import com.sos.jobnet.db.JobNetPlanDBItem;
import com.sos.jobnetmodel.objects.ExternalJobnetNodeObject;
import com.sos.jobnetmodel.objects.JobnetNode;
import com.sos.jobnetmodel.objects.JobnetNodeImpl;
import com.sos.jobnetmodel.objects.JobnetNodeType;
import com.sos.jobnetmodel.objects.NodeStatus;

/*
 * Adapter class to convert one node of the database representation of the jobnet into an internal jobnet node.
 */
public class JobNetPlanRecordAdapter extends JobnetNodeImpl implements ExternalJobnetNodeObject {
	
	private static Logger logger = Logger.getLogger(JobNetPlanRecordAdapter.class);
	
	private final JobNetEdgesDBLayer edgesDBReader;
	private final JobNetPlanDBItem dbRecord;
	
    private final List<String> successors;
    private final boolean isBootstrap;
    private final String uniqueName;
    
    private final static Pattern nodePattern = Pattern.compile("_step\\d\\d\\d");
	
	public JobNetPlanRecordAdapter(JobNetEdgesDBLayer edgesDBReader, JobNetPlanDBItem dbRecord) {
//		super( String.valueOf(dbRecord.getNodeId()), getType(dbRecord));
		super( String.valueOf(dbRecord.getNodeId()), getType2(dbRecord));
		this.dbRecord = dbRecord;
		this.edgesDBReader = edgesDBReader;
		this.successors = readSuccessors();
		this.isBootstrap = dbRecord.getBootstrap();
		this.uniqueName = dbRecord.getJobnetNodeDBItem().getNode();
		
		logger.debug(getType().name() + ": Adapt jobnet node " + dbRecord.getJobnetNodeDBItem().getNode() + " (id " + dbRecord.getNodeId() + ")");
        setSuccessors(successors);
        setExternalNodeObject(this);
        setName(dbRecord.getJobnetNodeDBItem().getNode());
        
        NodeStatus s = NodeStatus.valueOf(dbRecord.getStatusNode());
        setNodeStatus(s);
	}
	
	private static JobnetNodeType getType2(JobNetPlanDBItem dbRecord) {
		return JobnetNodeType.valueOf(dbRecord.getExitMessage());		// TODO eigenes DB Feld
	}
	
	
	//TODO expizite Schnellloesung fuer PF, der Typ muss aus dem Datensatz ermittelt werden.
	private static JobnetNodeType getType3(String name) {
		JobnetNodeType result = JobnetNodeType.HEAD;
		Matcher m = nodePattern.matcher(name);
		if (m.find()) {
			if(name.endsWith(m.group()))
				result = JobnetNodeType.CONNECTOR;
			else
				result = JobnetNodeType.JOB;
		}
		return result;
	}

	private List<String> readSuccessors() {
		List<String> result = new ArrayList<String>();
		List<JobNetEdgesDBItem> jobnetEdgesList = edgesDBReader.getSuccessors(dbRecord.getJobnetNodeDBItem());
		for(JobNetEdgesDBItem edge : jobnetEdgesList) {
			logger.debug("Successor " + edge.getChildNodeId() + " detected.");
			result.add( String.valueOf(edge.getChildNodeId()) );
		}
		return result;
	}

	@Override
	public List<String> getSuccessors() {
		return successors;
	}

	@Override
	public boolean hasSuccessors() {
		return (!successors.isEmpty());
	}

	@Override
	public boolean isBootstrap() {
		return isBootstrap;
	}
	
	@Override
	public String getUniqueName() {
		return uniqueName;
	}

	@Override
	public JobnetNode getJobnetNodeObject() {
		return this;
	}

}
