/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.engine.main;

import java.io.File;
import java.util.concurrent.atomic.AtomicReference;

import com.sos.scheduler.engine.kernel.CppScheduler;
import com.sos.scheduler.engine.kernel.scheduler.SchedulerException;

/** Der Scheduler in einem eigenen Thread. */
class SchedulerThread extends Thread {
    private final SchedulerControllerBridge controllerBridge;
    private final CppScheduler cppScheduler = new CppScheduler();
    private String[] arguments = {};
    private final AtomicReference<Integer> exitCodeAtom = new AtomicReference<Integer>();

    SchedulerThread(SchedulerControllerBridge controllerBridge) {
        this.controllerBridge = controllerBridge;
        setName("Scheduler");
    }

    final void loadModule(File f) {
        cppScheduler.loadModule(f);
    }

    final void startThread(String[] args) {
        this.arguments = args;
        start();  // Thread läuft in run()
    }

    @Override public final void run() {
        int exitCode = -1;
        Throwable throwable = null;
        try {
            exitCode = cppScheduler.run(arguments, "", controllerBridge);
            exitCodeAtom.set(exitCode);
            if (exitCode != 0)
                throwable = new SchedulerException("Scheduler terminated with exit code " + exitCode);
        }
        catch (Exception x) {
            throwable = x;
        }
        catch (Error x) {
            throwable = x;
            throw x;
        }
        finally {
            controllerBridge.onSchedulerTerminated(exitCode, throwable);
        }
    }

    final int exitCode() {
        if (isAlive())  throw new IllegalStateException("Thread is still alive");
        return exitCodeAtom.get();
    }
}
