/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: Order_queue.java 3946 2005-09-26 08:52:01Z jz $

package sos.spooler;

/*+ Auftragswarteschlange eines auftragsgesteuerten Jobs.
 * 
 * <p>
 * Ein auftragsgesteuerter Job (&lt;job order="yes">) hat eine Auftragswarteschlange, 
 * die die vom Job zu verarbeitenden Aufträge aufnimmt.
 * Die Aufträge sind nach Priorität und Zeitpunkt des Eintreffens geordnet.
 * <p>
 * Verarbeiten bedeutet, dass der Scheduler die Methode {@link Job_impl#spooler_process()} einer Task des Jobs aufruft.
 * Die Methode kann über spooler_task.order() auf den Auftrag zugreifen.
 * Endet spooler_process() ohne Fehler (ohne Exception), entfernt der Scheduler den Auftrag aus der Auftragswarteschlange.
 * Ist der Auftrag in einer Jobkette, dann rückt der rückt der Auftrag an die nächste Position der Jobkette.  
 * 
 * @see Order
 * @see Job_chain
 * @see Task#order() 
 */
/** 
 * @author Joacim Zschimmer
 * @version $Revision: 3946 $
 */

public class Order_queue extends Idispatch
{
    private                 Order_queue         ( long idispatch )                  { super(idispatch); }

    /*+ Liefert die Anzahl der Aufträge in der Auftragswarteschlange. */
    public int              length              ()                                  { return        int_com_call( "<length"             ); }
    
    
    
    /*+ Fügt einen Auftrag der Auftragswarteschlange hinzu.
     * <p>
     * Der Aufruf gilt nur für den Fall, dass der Auftrag nicht in einer Jobkette enthalten ist.
     * 
     * <p>
     * Die Priorität des Auftrags ({@link Order#set_priority(int)}) wird dabei berücksichtigt.
     * <p>
     * Wenn die Auftragswarteschlange bereits einen Auftrag mit gleicher Id enthält, wird dieser ersetzt.
     * 
     * @param order Der Auftrag
     * @see Job_chain#add_order(Order)
     */ 
    public Order            add_order           ( Order order )                     { return (Order)    com_call( "add_order", order    ); }
}
