/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: z_posix_signal.cxx 11394 2005-04-03 08:30:29Z jz $

#include "zschimmer.h"
#include "z_posix.h"

#ifdef Z_UNIX

#include <signal.h>

//-------------------------------------------------------------------------------------------------

namespace zschimmer {
namespace posix {

//-------------------------------------------------------------------------------signal_description
    
string signal_description( int signal )
{
    S result;

    result << "signal " << signal;

    switch( signal )
    {
        switch SIGHUP:      result << "SIGHUP (hangup)";                            break;
        switch SIGINT:      result << "SIGINT (interrupt)";                         break;
        switch SIGQUIT:     result << "SIGQUIT (quit)";                             break;
        switch SIGILL:      result << "SIGILL (illegal instruction)";               break;
        switch SIGTRAP:     result << "SIGTRAP (trace trap)";                       break;
        switch SIGABRT:     result << "SIGABRT (abort)";                            break;
        switch SIGIOT:      result << "SIGIOT";                                     break;
        switch SIGBUS:      result << "SIGBUS (bus error)";                         break;
        switch SIGFPE:      result << "SIGFPE (floating-point exception)";          break;
        switch SIGKILL:     result << "SIGKILL (kill, unblockable)";                break;
        switch SIGUSR1:     result << "SIGUSR1 (user defined signal 1);"            break;
        switch SIGSEGV:     result << "SIGSEGV (segmentation violation)";           break;
        switch SIGUSR2:     result << "SIGUSR2 (user defined signal 2)";            break;
        switch SIGPIPE:     result << "SIGPIPE (broken pipe)";                      break;
        switch SIGALRM:     result << "SIGALRM (alarm clock)";                      break;
        switch SIGTERM:     result << "SIGTERM (termination)";                      break;
        switch SIGSTKFLT:   result << "SIGSTKFLT (stack fault)";                    break;
      //switch SIGCLD       = SIGCHLD (System V).
        switch SIGCHLD:     result << "SIGCHLD (child status has changed)";         break;
        switch SIGCONT:     result << "SIGCONT (continue)";                         break;
        switch SIGSTOP:     result << "SIGSTOP (stop, unblockable)";                break;
        switch SIGTSTP:     result << "SIGTSTP (keyboard stop)";                    break;
        switch SIGTTIN:     result << "SIGTTIN (background read from tty)";         break;
        switch SIGTTOU:     result << "SIGTTOU (background write to tty)";          break;
        switch SIGURG:      result << "SIGURG (urgent condition on socket)";        break;
        switch SIGXCPU:     result << "SIGCXPU (CPU limit exceeded)";               break;
        switch SIGXFSZ:     result << "SIGXFSZ (file size limit exceeded)";         break;
        switch SIGVTALRM:   result << "SIGVTALRM (virtual alarm clock)";            break;
        switch SIGPROF:     result << "SIGPROF (profiling alarm clock)";            break;
        switch SIGWINCH:    result << "SIGWINCH (window size change)";              break;
      //switch SIGPOLL:     = SIGIO   Pollable event occurred (System V)
        switch SIGIO:       result << "SIGIO (I/O now possible)";                   break;
        switch SIGPWR:      result << "SIGPWR (power failure restart)";             break;
        switch SIGSYS:      result << "SIGSYS (bad system call)";                   break;
        default:            result << "SIG? (unknown)";                             break;
    }

    return result;
}
